"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from moovio_sdk import utils
from moovio_sdk._hooks import HookContext
from moovio_sdk.models import components, errors, operations
from moovio_sdk.types import OptionalNullable, UNSET
from moovio_sdk.utils import get_security_from_env
from moovio_sdk.utils.unmarshal_json_response import unmarshal_json_response
from typing import Any, Dict, List, Mapping, Optional


class Wallets(BaseSDK):
    def create(
        self,
        *,
        account_id: str,
        name: str,
        description: Optional[str] = None,
        metadata: Optional[Dict[str, str]] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.CreateWalletResponse:
        r"""Create a new wallet for an account. You can specify optional attributes such as a display name and description to specify the intended use of the wallet. This will generate a new moov-wallet payment method.

        Read our [Moov wallets guide](https://docs.moov.io/guides/sources/wallets/) to learn more.

        To access this endpoint using an [access token](https://docs.moov.io/api/authentication/access-tokens/)
        you'll need to specify the `/accounts/{accountID}/wallets.write` scope.

        :param account_id: The Moov account ID the wallet belongs to.
        :param name: Name of the wallet.
        :param description: Description of the wallet.
        :param metadata: Free-form key-value pair list. Useful for storing information that is not captured elsewhere.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.CreateWalletRequest(
            account_id=account_id,
            create_wallet=components.CreateWallet(
                name=name,
                description=description,
                metadata=metadata,
            ),
        )

        req = self._build_request(
            method="POST",
            path="/accounts/{accountID}/wallets",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=operations.CreateWalletGlobals(
                x_moov_version=self.sdk_configuration.globals.x_moov_version,
            ),
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.create_wallet, False, False, "json", components.CreateWallet
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="createWallet",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "403",
                "404",
                "409",
                "422",
                "429",
                "4XX",
                "500",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return operations.CreateWalletResponse(
                result=unmarshal_json_response(components.Wallet, http_res),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, ["400", "409"], "application/json"):
            response_data = unmarshal_json_response(errors.GenericErrorData, http_res)
            raise errors.GenericError(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.CreateWalletValidationErrorData, http_res
            )
            raise errors.CreateWalletValidationError(response_data, http_res)
        if utils.match_response(http_res, ["401", "403", "404", "429"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, ["500", "504"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    async def create_async(
        self,
        *,
        account_id: str,
        name: str,
        description: Optional[str] = None,
        metadata: Optional[Dict[str, str]] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.CreateWalletResponse:
        r"""Create a new wallet for an account. You can specify optional attributes such as a display name and description to specify the intended use of the wallet. This will generate a new moov-wallet payment method.

        Read our [Moov wallets guide](https://docs.moov.io/guides/sources/wallets/) to learn more.

        To access this endpoint using an [access token](https://docs.moov.io/api/authentication/access-tokens/)
        you'll need to specify the `/accounts/{accountID}/wallets.write` scope.

        :param account_id: The Moov account ID the wallet belongs to.
        :param name: Name of the wallet.
        :param description: Description of the wallet.
        :param metadata: Free-form key-value pair list. Useful for storing information that is not captured elsewhere.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.CreateWalletRequest(
            account_id=account_id,
            create_wallet=components.CreateWallet(
                name=name,
                description=description,
                metadata=metadata,
            ),
        )

        req = self._build_request_async(
            method="POST",
            path="/accounts/{accountID}/wallets",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=operations.CreateWalletGlobals(
                x_moov_version=self.sdk_configuration.globals.x_moov_version,
            ),
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.create_wallet, False, False, "json", components.CreateWallet
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="createWallet",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "403",
                "404",
                "409",
                "422",
                "429",
                "4XX",
                "500",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return operations.CreateWalletResponse(
                result=unmarshal_json_response(components.Wallet, http_res),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, ["400", "409"], "application/json"):
            response_data = unmarshal_json_response(errors.GenericErrorData, http_res)
            raise errors.GenericError(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.CreateWalletValidationErrorData, http_res
            )
            raise errors.CreateWalletValidationError(response_data, http_res)
        if utils.match_response(http_res, ["401", "403", "404", "429"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, ["500", "504"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    def list(
        self,
        *,
        account_id: str,
        status: Optional[components.WalletStatus] = None,
        wallet_type: Optional[components.WalletType] = None,
        skip: Optional[int] = None,
        count: Optional[int] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.ListWalletsResponse:
        r"""List the wallets associated with a Moov account.

        Read our [Moov wallets guide](https://docs.moov.io/guides/sources/wallets/) to learn more.

        To access this endpoint using an [access token](https://docs.moov.io/api/authentication/access-tokens/)
        you'll need to specify the `/accounts/{accountID}/wallets.read` scope.

        :param account_id:
        :param status: Optional parameter for filtering wallets by status.
        :param wallet_type: Optional parameter for filtering wallets by type.
        :param skip:
        :param count:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.ListWalletsRequest(
            status=status,
            wallet_type=wallet_type,
            skip=skip,
            count=count,
            account_id=account_id,
        )

        req = self._build_request(
            method="GET",
            path="/accounts/{accountID}/wallets",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=operations.ListWalletsGlobals(
                x_moov_version=self.sdk_configuration.globals.x_moov_version,
            ),
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="listWallets",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=["401", "403", "422", "429", "4XX", "500", "504", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return operations.ListWalletsResponse(
                result=unmarshal_json_response(List[components.Wallet], http_res),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.ListWalletsValidationErrorData, http_res
            )
            raise errors.ListWalletsValidationError(response_data, http_res)
        if utils.match_response(http_res, ["401", "403", "429"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, ["500", "504"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    async def list_async(
        self,
        *,
        account_id: str,
        status: Optional[components.WalletStatus] = None,
        wallet_type: Optional[components.WalletType] = None,
        skip: Optional[int] = None,
        count: Optional[int] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.ListWalletsResponse:
        r"""List the wallets associated with a Moov account.

        Read our [Moov wallets guide](https://docs.moov.io/guides/sources/wallets/) to learn more.

        To access this endpoint using an [access token](https://docs.moov.io/api/authentication/access-tokens/)
        you'll need to specify the `/accounts/{accountID}/wallets.read` scope.

        :param account_id:
        :param status: Optional parameter for filtering wallets by status.
        :param wallet_type: Optional parameter for filtering wallets by type.
        :param skip:
        :param count:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.ListWalletsRequest(
            status=status,
            wallet_type=wallet_type,
            skip=skip,
            count=count,
            account_id=account_id,
        )

        req = self._build_request_async(
            method="GET",
            path="/accounts/{accountID}/wallets",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=operations.ListWalletsGlobals(
                x_moov_version=self.sdk_configuration.globals.x_moov_version,
            ),
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="listWallets",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=["401", "403", "422", "429", "4XX", "500", "504", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return operations.ListWalletsResponse(
                result=unmarshal_json_response(List[components.Wallet], http_res),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.ListWalletsValidationErrorData, http_res
            )
            raise errors.ListWalletsValidationError(response_data, http_res)
        if utils.match_response(http_res, ["401", "403", "429"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, ["500", "504"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    def get(
        self,
        *,
        account_id: str,
        wallet_id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.GetWalletResponse:
        r"""Get information on a specific wallet (e.g., the available balance).

        Read our [Moov wallets guide](https://docs.moov.io/guides/sources/wallets/) to learn more.

        To access this endpoint using an [access token](https://docs.moov.io/api/authentication/access-tokens/)
        you'll need to specify the `/accounts/{accountID}/wallets.read` scope.

        :param account_id:
        :param wallet_id:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.GetWalletRequest(
            account_id=account_id,
            wallet_id=wallet_id,
        )

        req = self._build_request(
            method="GET",
            path="/accounts/{accountID}/wallets/{walletID}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=operations.GetWalletGlobals(
                x_moov_version=self.sdk_configuration.globals.x_moov_version,
            ),
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="getWallet",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=["401", "403", "404", "429", "4XX", "500", "504", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return operations.GetWalletResponse(
                result=unmarshal_json_response(components.Wallet, http_res),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, ["401", "403", "404", "429"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, ["500", "504"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    async def get_async(
        self,
        *,
        account_id: str,
        wallet_id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.GetWalletResponse:
        r"""Get information on a specific wallet (e.g., the available balance).

        Read our [Moov wallets guide](https://docs.moov.io/guides/sources/wallets/) to learn more.

        To access this endpoint using an [access token](https://docs.moov.io/api/authentication/access-tokens/)
        you'll need to specify the `/accounts/{accountID}/wallets.read` scope.

        :param account_id:
        :param wallet_id:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.GetWalletRequest(
            account_id=account_id,
            wallet_id=wallet_id,
        )

        req = self._build_request_async(
            method="GET",
            path="/accounts/{accountID}/wallets/{walletID}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=operations.GetWalletGlobals(
                x_moov_version=self.sdk_configuration.globals.x_moov_version,
            ),
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="getWallet",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=["401", "403", "404", "429", "4XX", "500", "504", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return operations.GetWalletResponse(
                result=unmarshal_json_response(components.Wallet, http_res),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, ["401", "403", "404", "429"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, ["500", "504"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    def update(
        self,
        *,
        wallet_id: str,
        account_id: str,
        name: Optional[str] = None,
        status: Optional[components.WalletStatus] = None,
        description: Optional[str] = None,
        metadata: Optional[Dict[str, str]] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.UpdateWalletResponse:
        r"""Update properties of an existing wallet such as name, description, status, or metadata.

        Read our [Moov wallets guide](https://docs.moov.io/guides/sources/wallets/) to learn more.

        To access this endpoint using an [access token](https://docs.moov.io/api/authentication/access-tokens/)
        you'll need to specify the `/accounts/{accountID}/wallets.write` scope.

        :param wallet_id: Identifier for the wallet.
        :param account_id: The Moov account ID the wallet belongs to.
        :param name:
        :param status: Status of a wallet.
            - `active`: The wallet is available for use and has an enabled payment method.
            - `closed`: The wallet is no longer active and the corresponding payment method has been disabled.
        :param description:
        :param metadata: Free-form key-value pair list. Useful for storing information that is not captured elsewhere.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.UpdateWalletRequest(
            wallet_id=wallet_id,
            account_id=account_id,
            patch_wallet=components.PatchWallet(
                name=name,
                status=status,
                description=description,
                metadata=metadata,
            ),
        )

        req = self._build_request(
            method="PATCH",
            path="/accounts/{accountID}/wallets/{walletID}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=operations.UpdateWalletGlobals(
                x_moov_version=self.sdk_configuration.globals.x_moov_version,
            ),
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.patch_wallet, False, False, "json", components.PatchWallet
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="updateWallet",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "403",
                "404",
                "409",
                "422",
                "429",
                "4XX",
                "500",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return operations.UpdateWalletResponse(
                result=unmarshal_json_response(components.Wallet, http_res),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, ["400", "409"], "application/json"):
            response_data = unmarshal_json_response(errors.GenericErrorData, http_res)
            raise errors.GenericError(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.PatchWalletValidationErrorData, http_res
            )
            raise errors.PatchWalletValidationError(response_data, http_res)
        if utils.match_response(http_res, ["401", "403", "404", "429"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, ["500", "504"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    async def update_async(
        self,
        *,
        wallet_id: str,
        account_id: str,
        name: Optional[str] = None,
        status: Optional[components.WalletStatus] = None,
        description: Optional[str] = None,
        metadata: Optional[Dict[str, str]] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.UpdateWalletResponse:
        r"""Update properties of an existing wallet such as name, description, status, or metadata.

        Read our [Moov wallets guide](https://docs.moov.io/guides/sources/wallets/) to learn more.

        To access this endpoint using an [access token](https://docs.moov.io/api/authentication/access-tokens/)
        you'll need to specify the `/accounts/{accountID}/wallets.write` scope.

        :param wallet_id: Identifier for the wallet.
        :param account_id: The Moov account ID the wallet belongs to.
        :param name:
        :param status: Status of a wallet.
            - `active`: The wallet is available for use and has an enabled payment method.
            - `closed`: The wallet is no longer active and the corresponding payment method has been disabled.
        :param description:
        :param metadata: Free-form key-value pair list. Useful for storing information that is not captured elsewhere.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.UpdateWalletRequest(
            wallet_id=wallet_id,
            account_id=account_id,
            patch_wallet=components.PatchWallet(
                name=name,
                status=status,
                description=description,
                metadata=metadata,
            ),
        )

        req = self._build_request_async(
            method="PATCH",
            path="/accounts/{accountID}/wallets/{walletID}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=operations.UpdateWalletGlobals(
                x_moov_version=self.sdk_configuration.globals.x_moov_version,
            ),
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.patch_wallet, False, False, "json", components.PatchWallet
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="updateWallet",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "403",
                "404",
                "409",
                "422",
                "429",
                "4XX",
                "500",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return operations.UpdateWalletResponse(
                result=unmarshal_json_response(components.Wallet, http_res),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, ["400", "409"], "application/json"):
            response_data = unmarshal_json_response(errors.GenericErrorData, http_res)
            raise errors.GenericError(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.PatchWalletValidationErrorData, http_res
            )
            raise errors.PatchWalletValidationError(response_data, http_res)
        if utils.match_response(http_res, ["401", "403", "404", "429"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, ["500", "504"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .authorizeduser import AuthorizedUser, AuthorizedUserTypedDict
from .cardbrand import CardBrand
from .cardexpiration import CardExpiration, CardExpirationTypedDict
from .issuedcardformfactor import IssuedCardFormFactor
from .issuedcardstate import IssuedCardState
from .issuingcontrols import IssuingControls, IssuingControlsTypedDict
from datetime import datetime
from moovio_sdk.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class FullIssuedCardTypedDict(TypedDict):
    r"""The full details of an issued card, including PAN and CVV."""

    issued_card_id: str
    brand: CardBrand
    r"""The card brand."""
    last_four_card_number: str
    expiration: CardExpirationTypedDict
    r"""The expiration date of the card or token."""
    authorized_user: AuthorizedUserTypedDict
    r"""Fields for identifying an authorized individual."""
    funding_wallet_id: str
    r"""Unique identifier for the wallet funding the card."""
    state: IssuedCardState
    r"""The `state` represents the operational status of an issued card. A card can only approve incoming authorizations if it is in an active state.

    - `active`: The card is operational and approves authorizations. Generally becomes active shortly after card creation.
    - `inactive`: The card cannot approve authorizations. This is currently a temporary state assigned post-creation during the activation process.
    - `closed`: The card is permanently deactivated and cannot approve authorizations. A card can be closed by request or when it expires.
    - `pending-verification`: Awaiting additional authorized user verification before the card can be activated.
    """
    form_factor: IssuedCardFormFactor
    r"""Specifies the type of spend card to be issued. Presently supports virtual only, providing a digital number without a physical card."""
    created_on: datetime
    pan: str
    r"""The issued card's Primary Account Number (PAN)"""
    cvv: str
    r"""The issued card's 3- or 4-digit Card Verification Value (CVV)."""
    memo: NotRequired[str]
    r"""Optional descriptor for the card."""
    controls: NotRequired[IssuingControlsTypedDict]


class FullIssuedCard(BaseModel):
    r"""The full details of an issued card, including PAN and CVV."""

    issued_card_id: Annotated[str, pydantic.Field(alias="issuedCardID")]

    brand: CardBrand
    r"""The card brand."""

    last_four_card_number: Annotated[str, pydantic.Field(alias="lastFourCardNumber")]

    expiration: CardExpiration
    r"""The expiration date of the card or token."""

    authorized_user: Annotated[AuthorizedUser, pydantic.Field(alias="authorizedUser")]
    r"""Fields for identifying an authorized individual."""

    funding_wallet_id: Annotated[str, pydantic.Field(alias="fundingWalletID")]
    r"""Unique identifier for the wallet funding the card."""

    state: IssuedCardState
    r"""The `state` represents the operational status of an issued card. A card can only approve incoming authorizations if it is in an active state.

    - `active`: The card is operational and approves authorizations. Generally becomes active shortly after card creation.
    - `inactive`: The card cannot approve authorizations. This is currently a temporary state assigned post-creation during the activation process.
    - `closed`: The card is permanently deactivated and cannot approve authorizations. A card can be closed by request or when it expires.
    - `pending-verification`: Awaiting additional authorized user verification before the card can be activated.
    """

    form_factor: Annotated[IssuedCardFormFactor, pydantic.Field(alias="formFactor")]
    r"""Specifies the type of spend card to be issued. Presently supports virtual only, providing a digital number without a physical card."""

    created_on: Annotated[datetime, pydantic.Field(alias="createdOn")]

    pan: str
    r"""The issued card's Primary Account Number (PAN)"""

    cvv: str
    r"""The issued card's 3- or 4-digit Card Verification Value (CVV)."""

    memo: Optional[str] = None
    r"""Optional descriptor for the card."""

    controls: Optional[IssuingControls] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["memo", "controls"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    FullIssuedCard.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from moovio_sdk.types import BaseModel
from typing_extensions import TypedDict


class AccountCapabilityTypedDict(TypedDict):
    r"""Describes an action or set of actions that an account is permitted to perform."""

    capability: str
    status: str


class AccountCapability(BaseModel):
    r"""Describes an action or set of actions that an account is permitted to perform."""

    capability: str

    status: str



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .calltoaction import CallToAction
from moovio_sdk.types import BaseModel
import pydantic
from typing_extensions import Annotated, TypedDict


class PaymentLinkDisplayOptionsTypedDict(TypedDict):
    r"""Customizable display options for a payment link."""

    title: str
    r"""The payment page title displayed to the user."""
    description: str
    r"""A payment description displayed to the user."""
    call_to_action: CallToAction
    r"""The text to be displayed on web form's submit button.

    If set to \"auto\" the UI will automatically select between
    \"pay\" and \"confirm\" for payments and payouts respectively.
    """


class PaymentLinkDisplayOptions(BaseModel):
    r"""Customizable display options for a payment link."""

    title: str
    r"""The payment page title displayed to the user."""

    description: str
    r"""A payment description displayed to the user."""

    call_to_action: Annotated[CallToAction, pydantic.Field(alias="callToAction")]
    r"""The text to be displayed on web form's submit button.

    If set to \"auto\" the UI will automatically select between
    \"pay\" and \"confirm\" for payments and payouts respectively.
    """


try:
    PaymentLinkDisplayOptions.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .cardbrandfees import CardBrandFees, CardBrandFeesTypedDict
from moovio_sdk.types import BaseModel
import pydantic
from typing_extensions import Annotated, TypedDict


class CardAcquiringFeesTypedDict(TypedDict):
    r"""A detailed breakdown of card acquiring fees by card brand."""

    visa: CardBrandFeesTypedDict
    r"""Fees associated with Visa transactions."""
    mastercard: CardBrandFeesTypedDict
    r"""Fees associated with Mastercard transactions."""
    discover: CardBrandFeesTypedDict
    r"""Fees associated with Discover transactions."""
    american_express: CardBrandFeesTypedDict
    r"""Fees associated with American Express transactions."""


class CardAcquiringFees(BaseModel):
    r"""A detailed breakdown of card acquiring fees by card brand."""

    visa: CardBrandFees
    r"""Fees associated with Visa transactions."""

    mastercard: CardBrandFees
    r"""Fees associated with Mastercard transactions."""

    discover: CardBrandFees
    r"""Fees associated with Discover transactions."""

    american_express: Annotated[CardBrandFees, pydantic.Field(alias="americanExpress")]
    r"""Fees associated with American Express transactions."""


try:
    CardAcquiringFees.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .transferpaymentmethodswallet import (
    TransferPaymentMethodsWallet,
    TransferPaymentMethodsWalletTypedDict,
)
from enum import Enum
from moovio_sdk.types import BaseModel
import pydantic
from typing_extensions import Annotated, TypedDict


class MoovWalletTransferPaymentMethodPaymentMethodType(str, Enum):
    MOOV_WALLET = "moov-wallet"


class MoovWalletTransferPaymentMethodTypedDict(TypedDict):
    payment_method_id: str
    r"""ID of the payment method."""
    payment_method_type: MoovWalletTransferPaymentMethodPaymentMethodType
    wallet: TransferPaymentMethodsWalletTypedDict


class MoovWalletTransferPaymentMethod(BaseModel):
    payment_method_id: Annotated[str, pydantic.Field(alias="paymentMethodID")]
    r"""ID of the payment method."""

    payment_method_type: Annotated[
        MoovWalletTransferPaymentMethodPaymentMethodType,
        pydantic.Field(alias="paymentMethodType"),
    ]

    wallet: TransferPaymentMethodsWallet


try:
    MoovWalletTransferPaymentMethod.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class WalletTransactionType(str, Enum):
    ACCOUNT_FUNDING = "account-funding"
    ACH_REVERSAL = "ach-reversal"
    AUTO_SWEEP = "auto-sweep"
    CARD_PAYMENT = "card-payment"
    CARD_DECLINE = "card-decline"
    CARD_REVERSAL = "card-reversal"
    CASH_OUT = "cash-out"
    DISPUTE = "dispute"
    DISPUTE_REVERSAL = "dispute-reversal"
    FACILITATOR_FEE = "facilitator-fee"
    ISSUING_REFUND = "issuing-refund"
    ISSUING_TRANSACTION = "issuing-transaction"
    ISSUING_TRANSACTION_ADJUSTMENT = "issuing-transaction-adjustment"
    ISSUING_AUTH_HOLD = "issuing-auth-hold"
    ISSUING_AUTH_RELEASE = "issuing-auth-release"
    ISSUING_DECLINE = "issuing-decline"
    MOOV_FEE = "moov-fee"
    PAYMENT = "payment"
    PAYOUT = "payout"
    REFUND = "refund"
    REFUND_FAILURE = "refund-failure"
    RTP_FAILURE = "rtp-failure"
    TOP_UP = "top-up"
    WALLET_TRANSFER = "wallet-transfer"
    ADJUSTMENT = "adjustment"
    FEE_REVENUE = "fee-revenue"
    RESIDUAL = "residual"
    INSTANT_BANK_FAILURE = "instant-bank-failure"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from dataclasses import dataclass, field
import httpx
from moovio_sdk.models.errors import MoovError
from moovio_sdk.types import BaseModel
import pydantic
from typing import Optional
from typing_extensions import Annotated


class DuplicateCardErrorData(BaseModel):
    error: str
    r"""A human-readable description of the error."""
    existing_card_id: Annotated[
        Optional[str], pydantic.Field(alias="existingCardID")
    ] = None
    r"""The ID of the existing card that caused the conflict.
    This field is included when attempting to link a card that already exists on the account.
    """


@dataclass(unsafe_hash=True)
class DuplicateCardError(MoovError):
    data: DuplicateCardErrorData = field(hash=False)

    def __init__(
        self,
        data: DuplicateCardErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        message = body or raw_response.text
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .invoicelineitem import InvoiceLineItem, InvoiceLineItemTypedDict
from moovio_sdk.types import BaseModel
from typing import List
from typing_extensions import TypedDict


class InvoiceLineItemsTypedDict(TypedDict):
    r"""A collection of line items for an invoice."""

    items: List[InvoiceLineItemTypedDict]
    r"""The list of line items."""


class InvoiceLineItems(BaseModel):
    r"""A collection of line items for an invoice."""

    items: List[InvoiceLineItem]
    r"""The list of line items."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .createpaymentlinklineitem import (
    CreatePaymentLinkLineItem,
    CreatePaymentLinkLineItemTypedDict,
)
from moovio_sdk.types import BaseModel
from typing import List
from typing_extensions import TypedDict


class CreatePaymentLinkLineItemsTypedDict(TypedDict):
    r"""An optional collection of line items for a payment link.
    When line items are provided, their total plus sales tax must equal the payment link amount.
    """

    items: List[CreatePaymentLinkLineItemTypedDict]
    r"""The list of line items."""


class CreatePaymentLinkLineItems(BaseModel):
    r"""An optional collection of line items for a payment link.
    When line items are provided, their total plus sales tax must equal the payment link amount.
    """

    items: List[CreatePaymentLinkLineItem]
    r"""The list of line items."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from datetime import datetime
from moovio_sdk.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class SweepConfigPaymentMethodTypedDict(TypedDict):
    r"""The payment method used to push or pull funds to a bank account.
    The push payment method can only be ach-credit-standard, ach-credit-same-day, or rtp-credit. The pull payment method can only be ach-debit-fund.
    """

    payment_method_id: str
    r"""ID of the payment method."""
    disabled_on: NotRequired[datetime]


class SweepConfigPaymentMethod(BaseModel):
    r"""The payment method used to push or pull funds to a bank account.
    The push payment method can only be ach-credit-standard, ach-credit-same-day, or rtp-credit. The pull payment method can only be ach-debit-fund.
    """

    payment_method_id: Annotated[str, pydantic.Field(alias="paymentMethodID")]
    r"""ID of the payment method."""

    disabled_on: Annotated[Optional[datetime], pydantic.Field(alias="disabledOn")] = (
        None
    )

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["disabledOn"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    SweepConfigPaymentMethod.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from dataclasses import dataclass, field
import httpx
from moovio_sdk.models.components import (
    cardvolumedistributionerror as components_cardvolumedistributionerror,
    fulfillmentdetailserror as components_fulfillmentdetailserror,
    volumebycustomertypeerror as components_volumebycustomertypeerror,
)
from moovio_sdk.models.errors import MoovError
from moovio_sdk.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class UpdateUnderwritingErrorErrorTypedDict(TypedDict):
    average_monthly_transaction_volume: NotRequired[str]
    average_transaction_size: NotRequired[str]
    max_transaction_size: NotRequired[str]
    volume_by_customer_type: NotRequired[
        components_volumebycustomertypeerror.VolumeByCustomerTypeErrorTypedDict
    ]
    card_volume_distribution: NotRequired[
        components_cardvolumedistributionerror.CardVolumeDistributionErrorTypedDict
    ]
    fulfillment: NotRequired[
        components_fulfillmentdetailserror.FulfillmentDetailsErrorTypedDict
    ]


class UpdateUnderwritingErrorError(BaseModel):
    average_monthly_transaction_volume: Annotated[
        Optional[str], pydantic.Field(alias="averageMonthlyTransactionVolume")
    ] = None

    average_transaction_size: Annotated[
        Optional[str], pydantic.Field(alias="averageTransactionSize")
    ] = None

    max_transaction_size: Annotated[
        Optional[str], pydantic.Field(alias="maxTransactionSize")
    ] = None

    volume_by_customer_type: Annotated[
        Optional[components_volumebycustomertypeerror.VolumeByCustomerTypeError],
        pydantic.Field(alias="volumeByCustomerType"),
    ] = None

    card_volume_distribution: Annotated[
        Optional[components_cardvolumedistributionerror.CardVolumeDistributionError],
        pydantic.Field(alias="cardVolumeDistribution"),
    ] = None

    fulfillment: Optional[
        components_fulfillmentdetailserror.FulfillmentDetailsError
    ] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "averageMonthlyTransactionVolume",
                "averageTransactionSize",
                "maxTransactionSize",
                "volumeByCustomerType",
                "cardVolumeDistribution",
                "fulfillment",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class UpdateUnderwritingErrorData(BaseModel):
    error: UpdateUnderwritingErrorError


@dataclass(unsafe_hash=True)
class UpdateUnderwritingError(MoovError):
    data: UpdateUnderwritingErrorData = field(hash=False)

    def __init__(
        self,
        data: UpdateUnderwritingErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        message = body or raw_response.text
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


try:
    UpdateUnderwritingErrorError.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .createtransferachaddendarecord import (
    CreateTransferACHAddendaRecord,
    CreateTransferACHAddendaRecordTypedDict,
)
from .debitholdperiod import DebitHoldPeriod
from .seccode import SECCode
from moovio_sdk.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class CreateTransferSourceACHTypedDict(TypedDict):
    company_entry_description: NotRequired[str]
    r"""An optional override of the default NACHA company entry description for a transfer."""
    originating_company_name: NotRequired[str]
    r"""An optional override of the default NACHA company name for a transfer."""
    debit_hold_period: NotRequired[DebitHoldPeriod]
    r"""An optional override of your default ACH hold period in banking days. The hold period must be longer than or equal to your default setting."""
    sec_code: NotRequired[SECCode]
    r"""Code used to identify the ACH authorization method."""
    addenda: NotRequired[List[CreateTransferACHAddendaRecordTypedDict]]


class CreateTransferSourceACH(BaseModel):
    company_entry_description: Annotated[
        Optional[str], pydantic.Field(alias="companyEntryDescription")
    ] = None
    r"""An optional override of the default NACHA company entry description for a transfer."""

    originating_company_name: Annotated[
        Optional[str], pydantic.Field(alias="originatingCompanyName")
    ] = None
    r"""An optional override of the default NACHA company name for a transfer."""

    debit_hold_period: Annotated[
        Optional[DebitHoldPeriod], pydantic.Field(alias="debitHoldPeriod")
    ] = None
    r"""An optional override of your default ACH hold period in banking days. The hold period must be longer than or equal to your default setting."""

    sec_code: Annotated[Optional[SECCode], pydantic.Field(alias="secCode")] = None
    r"""Code used to identify the ACH authorization method."""

    addenda: Optional[List[CreateTransferACHAddendaRecord]] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "companyEntryDescription",
                "originatingCompanyName",
                "debitHoldPeriod",
                "secCode",
                "addenda",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    CreateTransferSourceACH.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .amountdecimal import AmountDecimal, AmountDecimalTypedDict
from .invoicelineitems import InvoiceLineItems, InvoiceLineItemsTypedDict
from .invoicepayment import InvoicePayment, InvoicePaymentTypedDict
from .invoicestatus import InvoiceStatus
from datetime import datetime
from moovio_sdk.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class InvoiceTypedDict(TypedDict):
    invoice_id: str
    r"""A unique identifier for a Moov resource. Supports UUID format (xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx) or typed format with base32-encoded UUID and type suffix (e.g., kuoaydiojf7uszaokc2ggnaaaa_xfer)."""
    invoice_number: str
    customer_account_id: str
    r"""A unique identifier for a Moov resource. Supports UUID format (xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx) or typed format with base32-encoded UUID and type suffix (e.g., kuoaydiojf7uszaokc2ggnaaaa_xfer)."""
    customer_display_name: str
    r"""Display name of the customer account."""
    customer_email: str
    r"""Email address of the customer account."""
    partner_account_id: str
    r"""A unique identifier for a Moov resource. Supports UUID format (xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx) or typed format with base32-encoded UUID and type suffix (e.g., kuoaydiojf7uszaokc2ggnaaaa_xfer)."""
    status: InvoiceStatus
    line_items: InvoiceLineItemsTypedDict
    r"""A collection of line items for an invoice."""
    subtotal_amount: AmountDecimalTypedDict
    tax_amount: AmountDecimalTypedDict
    total_amount: AmountDecimalTypedDict
    r"""Total amount of the invoice, sum of subTotalAmount and taxAmount"""
    pending_amount: AmountDecimalTypedDict
    r"""Total amount of pending transfers paid towards the invoice"""
    paid_amount: AmountDecimalTypedDict
    r"""Total amount of completed transfers paid towards the invoice"""
    refunded_amount: AmountDecimalTypedDict
    r"""Total amount of refunds initiated against transfers paid towards the invoice"""
    disputed_amount: AmountDecimalTypedDict
    r"""Total amount of disputes initiated against transfers paid towards the invoice"""
    created_on: datetime
    description: NotRequired[str]
    payment_link_code: NotRequired[str]
    invoice_payments: NotRequired[List[InvoicePaymentTypedDict]]
    r"""Payment made towards an invoice, will be either a transfer or an external payment."""
    invoice_date: NotRequired[datetime]
    due_date: NotRequired[datetime]
    sent_on: NotRequired[datetime]
    paid_on: NotRequired[datetime]
    canceled_on: NotRequired[datetime]
    disabled_on: NotRequired[datetime]


class Invoice(BaseModel):
    invoice_id: Annotated[str, pydantic.Field(alias="invoiceID")]
    r"""A unique identifier for a Moov resource. Supports UUID format (xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx) or typed format with base32-encoded UUID and type suffix (e.g., kuoaydiojf7uszaokc2ggnaaaa_xfer)."""

    invoice_number: Annotated[str, pydantic.Field(alias="invoiceNumber")]

    customer_account_id: Annotated[str, pydantic.Field(alias="customerAccountID")]
    r"""A unique identifier for a Moov resource. Supports UUID format (xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx) or typed format with base32-encoded UUID and type suffix (e.g., kuoaydiojf7uszaokc2ggnaaaa_xfer)."""

    customer_display_name: Annotated[str, pydantic.Field(alias="customerDisplayName")]
    r"""Display name of the customer account."""

    customer_email: Annotated[str, pydantic.Field(alias="customerEmail")]
    r"""Email address of the customer account."""

    partner_account_id: Annotated[str, pydantic.Field(alias="partnerAccountID")]
    r"""A unique identifier for a Moov resource. Supports UUID format (xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx) or typed format with base32-encoded UUID and type suffix (e.g., kuoaydiojf7uszaokc2ggnaaaa_xfer)."""

    status: InvoiceStatus

    line_items: Annotated[InvoiceLineItems, pydantic.Field(alias="lineItems")]
    r"""A collection of line items for an invoice."""

    subtotal_amount: Annotated[AmountDecimal, pydantic.Field(alias="subtotalAmount")]

    tax_amount: Annotated[AmountDecimal, pydantic.Field(alias="taxAmount")]

    total_amount: Annotated[AmountDecimal, pydantic.Field(alias="totalAmount")]
    r"""Total amount of the invoice, sum of subTotalAmount and taxAmount"""

    pending_amount: Annotated[AmountDecimal, pydantic.Field(alias="pendingAmount")]
    r"""Total amount of pending transfers paid towards the invoice"""

    paid_amount: Annotated[AmountDecimal, pydantic.Field(alias="paidAmount")]
    r"""Total amount of completed transfers paid towards the invoice"""

    refunded_amount: Annotated[AmountDecimal, pydantic.Field(alias="refundedAmount")]
    r"""Total amount of refunds initiated against transfers paid towards the invoice"""

    disputed_amount: Annotated[AmountDecimal, pydantic.Field(alias="disputedAmount")]
    r"""Total amount of disputes initiated against transfers paid towards the invoice"""

    created_on: Annotated[datetime, pydantic.Field(alias="createdOn")]

    description: Optional[str] = None

    payment_link_code: Annotated[
        Optional[str], pydantic.Field(alias="paymentLinkCode")
    ] = None

    invoice_payments: Annotated[
        Optional[List[InvoicePayment]], pydantic.Field(alias="invoicePayments")
    ] = None
    r"""Payment made towards an invoice, will be either a transfer or an external payment."""

    invoice_date: Annotated[Optional[datetime], pydantic.Field(alias="invoiceDate")] = (
        None
    )

    due_date: Annotated[Optional[datetime], pydantic.Field(alias="dueDate")] = None

    sent_on: Annotated[Optional[datetime], pydantic.Field(alias="sentOn")] = None

    paid_on: Annotated[Optional[datetime], pydantic.Field(alias="paidOn")] = None

    canceled_on: Annotated[Optional[datetime], pydantic.Field(alias="canceledOn")] = (
        None
    )

    disabled_on: Annotated[Optional[datetime], pydantic.Field(alias="disabledOn")] = (
        None
    )

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "description",
                "paymentLinkCode",
                "invoicePayments",
                "invoiceDate",
                "dueDate",
                "sentOn",
                "paidOn",
                "canceledOn",
                "disabledOn",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    Invoice.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .disputephase import DisputePhase
from .disputestatus import DisputeStatus
from moovio_sdk.types import BaseModel
import pydantic
from typing_extensions import Annotated, TypedDict


class WebhookDataDisputeCreatedTypedDict(TypedDict):
    account_id: str
    transfer_id: str
    transaction_id: str
    dispute_id: str
    status: DisputeStatus
    r"""The status of a particular dispute.

    Read our [disputes guide](https://docs.moov.io/guides/money-movement/accept-payments/card-acceptance/disputes/#dispute-statuses) to learn what each status means.
    """
    phase: DisputePhase
    r"""The phase of a dispute within the dispute lifecycle."""


class WebhookDataDisputeCreated(BaseModel):
    account_id: Annotated[str, pydantic.Field(alias="accountID")]

    transfer_id: Annotated[str, pydantic.Field(alias="transferID")]

    transaction_id: Annotated[str, pydantic.Field(alias="transactionID")]

    dispute_id: Annotated[str, pydantic.Field(alias="disputeID")]

    status: DisputeStatus
    r"""The status of a particular dispute.

    Read our [disputes guide](https://docs.moov.io/guides/money-movement/accept-payments/card-acceptance/disputes/#dispute-statuses) to learn what each status means.
    """

    phase: DisputePhase
    r"""The phase of a dispute within the dispute lifecycle."""


try:
    WebhookDataDisputeCreated.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from moovio_sdk.types import BaseModel
from typing_extensions import TypedDict


class CreateTransferACHAddendaRecordTypedDict(TypedDict):
    r"""An addenda record to be added to an ACH transfer"""

    record: str
    r"""The raw ACH addenda record. Must only contain valid NACHA characters"""


class CreateTransferACHAddendaRecord(BaseModel):
    r"""An addenda record to be added to an ACH transfer"""

    record: str
    r"""The raw ACH addenda record. Must only contain valid NACHA characters"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from moovio_sdk.types import BaseModel
import pydantic
from typing_extensions import Annotated, TypedDict


class CreateApplePaySessionTypedDict(TypedDict):
    domain: str
    r"""A fully qualified top-level or sub-domain name where you will accept Apple Pay. Should not include \"https\"."""
    display_name: str
    r"""A UTF-8 string to display in the Buy button."""


class CreateApplePaySession(BaseModel):
    domain: str
    r"""A fully qualified top-level or sub-domain name where you will accept Apple Pay. Should not include \"https\"."""

    display_name: Annotated[str, pydantic.Field(alias="displayName")]
    r"""A UTF-8 string to display in the Buy button."""


try:
    CreateApplePaySession.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from moovio_sdk.models.components import (
    enrichedaddressresponse as components_enrichedaddressresponse,
)
from moovio_sdk.types import BaseModel, UNSET_SENTINEL
from moovio_sdk.utils import FieldMetadata, HeaderMetadata, QueryParamMetadata
import pydantic
from pydantic import model_serializer
from typing import Dict, List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class GetEnrichmentAddressGlobalsTypedDict(TypedDict):
    x_moov_version: NotRequired[str]
    r"""Specify an API version.

    API versioning follows the format `vYYYY.QQ.BB`, where
    - `YYYY` is the year
    - `QQ` is the two-digit month for the first month of the quarter (e.g., 01, 04, 07, 10)
    - `BB` is the build number, starting at `.01`, for subsequent builds in the same quarter.
    - For example, `v2024.01.00` is the initial release of the first quarter of 2024.

    The `dev` version represents the most recent development state. It may include breaking changes and should be treated as a beta release.
    When no version is specified, the API defaults to `v2024.01.00`.
    """


class GetEnrichmentAddressGlobals(BaseModel):
    x_moov_version: Annotated[
        Optional[str],
        pydantic.Field(alias="X-Moov-Version"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""Specify an API version.

    API versioning follows the format `vYYYY.QQ.BB`, where
    - `YYYY` is the year
    - `QQ` is the two-digit month for the first month of the quarter (e.g., 01, 04, 07, 10)
    - `BB` is the build number, starting at `.01`, for subsequent builds in the same quarter.
    - For example, `v2024.01.00` is the initial release of the first quarter of 2024.

    The `dev` version represents the most recent development state. It may include breaking changes and should be treated as a beta release.
    When no version is specified, the API defaults to `v2024.01.00`.
    """

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["X-Moov-Version"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class GetEnrichmentAddressRequestTypedDict(TypedDict):
    search: str
    r"""Partial or complete address to search."""
    max_results: NotRequired[int]
    r"""Maximum number of results to return."""
    include_cities: NotRequired[str]
    r"""Limits results to a list of given cities."""
    include_states: NotRequired[str]
    r"""Limits results to a list of given states."""
    include_zipcodes: NotRequired[str]
    r"""Limits results to a list of given zipcodes."""
    exclude_states: NotRequired[str]
    r"""Exclude list of states from results. No `include` pararmeters may be used with this parameter."""
    prefer_cities: NotRequired[str]
    r"""Display results with the listed cities at the top."""
    prefer_states: NotRequired[str]
    r"""Display results with the listed states at the top."""
    prefer_zipcodes: NotRequired[str]
    r"""Display results with the listed zipcodes at the top."""
    prefer_ratio: NotRequired[int]
    r"""Specifies the percentage of address suggestions that should be preferred and will appear at the top of the results."""
    prefer_geolocation: NotRequired[str]
    r"""If omitted or set to `city`, it uses the sender's IP address to determine location, then automatically adds the city and state
    to the preferCities value. This parameter takes precedence over other `include` or `exclude` parameters meaning that if it is
    not set to `none`, you may see addresses from areas you do not wish to see.
    """
    selected: NotRequired[str]
    r"""Useful for narrowing results with `addressLine2` suggestions such as `Apt` (denotes an apartment building with multiple residences)."""
    source: NotRequired[str]
    r"""Include results from alternate data sources. Allowed values are `all` (non-postal addresses), or `postal` (postal addresses only)."""


class GetEnrichmentAddressRequest(BaseModel):
    search: Annotated[
        str, FieldMetadata(query=QueryParamMetadata(style="form", explode=False))
    ]
    r"""Partial or complete address to search."""

    max_results: Annotated[
        Optional[int],
        pydantic.Field(alias="maxResults"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=False)),
    ] = None
    r"""Maximum number of results to return."""

    include_cities: Annotated[
        Optional[str],
        pydantic.Field(alias="includeCities"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=False)),
    ] = None
    r"""Limits results to a list of given cities."""

    include_states: Annotated[
        Optional[str],
        pydantic.Field(alias="includeStates"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=False)),
    ] = None
    r"""Limits results to a list of given states."""

    include_zipcodes: Annotated[
        Optional[str],
        pydantic.Field(alias="includeZipcodes"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=False)),
    ] = None
    r"""Limits results to a list of given zipcodes."""

    exclude_states: Annotated[
        Optional[str],
        pydantic.Field(alias="excludeStates"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=False)),
    ] = None
    r"""Exclude list of states from results. No `include` pararmeters may be used with this parameter."""

    prefer_cities: Annotated[
        Optional[str],
        pydantic.Field(alias="preferCities"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=False)),
    ] = None
    r"""Display results with the listed cities at the top."""

    prefer_states: Annotated[
        Optional[str],
        pydantic.Field(alias="preferStates"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=False)),
    ] = None
    r"""Display results with the listed states at the top."""

    prefer_zipcodes: Annotated[
        Optional[str],
        pydantic.Field(alias="preferZipcodes"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=False)),
    ] = None
    r"""Display results with the listed zipcodes at the top."""

    prefer_ratio: Annotated[
        Optional[int],
        pydantic.Field(alias="preferRatio"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=False)),
    ] = None
    r"""Specifies the percentage of address suggestions that should be preferred and will appear at the top of the results."""

    prefer_geolocation: Annotated[
        Optional[str],
        pydantic.Field(alias="preferGeolocation"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=False)),
    ] = None
    r"""If omitted or set to `city`, it uses the sender's IP address to determine location, then automatically adds the city and state
    to the preferCities value. This parameter takes precedence over other `include` or `exclude` parameters meaning that if it is
    not set to `none`, you may see addresses from areas you do not wish to see.
    """

    selected: Annotated[
        Optional[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=False)),
    ] = None
    r"""Useful for narrowing results with `addressLine2` suggestions such as `Apt` (denotes an apartment building with multiple residences)."""

    source: Annotated[
        Optional[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=False)),
    ] = None
    r"""Include results from alternate data sources. Allowed values are `all` (non-postal addresses), or `postal` (postal addresses only)."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "maxResults",
                "includeCities",
                "includeStates",
                "includeZipcodes",
                "excludeStates",
                "preferCities",
                "preferStates",
                "preferZipcodes",
                "preferRatio",
                "preferGeolocation",
                "selected",
                "source",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class GetEnrichmentAddressResponseTypedDict(TypedDict):
    headers: Dict[str, List[str]]
    result: components_enrichedaddressresponse.EnrichedAddressResponseTypedDict


class GetEnrichmentAddressResponse(BaseModel):
    headers: Dict[str, List[str]]

    result: components_enrichedaddressresponse.EnrichedAddressResponse



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .createproductoption import CreateProductOption, CreateProductOptionTypedDict
from moovio_sdk.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class CreateProductOptionGroupTypedDict(TypedDict):
    name: str
    min_select: int
    r"""The minimum number of options that must be selected from this group.

    A value of 0 indicates that no selection from this group is required.
    """
    max_select: int
    r"""The maximum number of options that can be selected from this group."""
    options: List[CreateProductOptionTypedDict]
    r"""The options available within this group."""
    description: NotRequired[str]
    r"""A detailed description of the option group.

    - Must be valid UTF-8 text
    - Supports Markdown for formatting
    - HTML is not permitted and will be rejected
    """


class CreateProductOptionGroup(BaseModel):
    name: str

    min_select: Annotated[int, pydantic.Field(alias="minSelect")]
    r"""The minimum number of options that must be selected from this group.

    A value of 0 indicates that no selection from this group is required.
    """

    max_select: Annotated[int, pydantic.Field(alias="maxSelect")]
    r"""The maximum number of options that can be selected from this group."""

    options: List[CreateProductOption]
    r"""The options available within this group."""

    description: Optional[str] = None
    r"""A detailed description of the option group.

    - Must be valid UTF-8 text
    - Supports Markdown for formatting
    - HTML is not permitted and will be rejected
    """

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["description"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    CreateProductOptionGroup.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .capabilityid import CapabilityID
from moovio_sdk.types import BaseModel
from typing import List
from typing_extensions import TypedDict


class AddCapabilitiesTypedDict(TypedDict):
    capabilities: List[CapabilityID]


class AddCapabilities(BaseModel):
    capabilities: List[CapabilityID]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from moovio_sdk.types import BaseModel
import pydantic
from typing_extensions import Annotated, TypedDict


class WebhookDataBankAccountCreatedTypedDict(TypedDict):
    bank_account_id: str
    account_id: str


class WebhookDataBankAccountCreated(BaseModel):
    bank_account_id: Annotated[str, pydantic.Field(alias="bankAccountID")]

    account_id: Annotated[str, pydantic.Field(alias="accountID")]


try:
    WebhookDataBankAccountCreated.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class BankAccountStatusReason(str, Enum):
    r"""The reason the bank account status changed to the current value."""

    BANK_ACCOUNT_CREATED = "bank-account-created"
    VERIFICATION_INITIATED = "verification-initiated"
    MICRO_DEPOSIT_ATTEMPTS_EXCEEDED = "micro-deposit-attempts-exceeded"
    MICRO_DEPOSIT_EXPIRED = "micro-deposit-expired"
    MAX_VERIFICATION_FAILURES = "max-verification-failures"
    VERIFICATION_ATTEMPTS_EXCEEDED = "verification-attempts-exceeded"
    VERIFICATION_EXPIRED = "verification-expired"
    VERIFICATION_SUCCESSFUL = "verification-successful"
    ACH_DEBIT_RETURN = "ach-debit-return"
    ACH_CREDIT_RETURN = "ach-credit-return"
    RTP_CREDIT_FAILURE = "rtp-credit-failure"
    FEDNOW_CREDIT_FAILURE = "fednow-credit-failure"
    MICRO_DEPOSIT_RETURN = "micro-deposit-return"
    ADMIN_ACTION = "admin-action"
    OTHER = "other"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class BusinessType(str, Enum):
    r"""The type of entity represented by this business."""

    SOLE_PROPRIETORSHIP = "soleProprietorship"
    UNINCORPORATED_ASSOCIATION = "unincorporatedAssociation"
    TRUST = "trust"
    PUBLIC_CORPORATION = "publicCorporation"
    PRIVATE_CORPORATION = "privateCorporation"
    LLC = "llc"
    PARTNERSHIP = "partnership"
    UNINCORPORATED_NON_PROFIT = "unincorporatedNonProfit"
    INCORPORATED_NON_PROFIT = "incorporatedNonProfit"
    GOVERNMENT_ENTITY = "governmentEntity"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .address import Address, AddressTypedDict
from .businesstype import BusinessType
from .industrycodes import IndustryCodes, IndustryCodesTypedDict
from .phonenumber import PhoneNumber, PhoneNumberTypedDict
from .primaryregulator import PrimaryRegulator
from .taxid import TaxID, TaxIDTypedDict
from moovio_sdk.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class CreateBusinessProfileTypedDict(TypedDict):
    legal_business_name: str
    r"""The legal name under which the entity is registered."""
    doing_business_as: NotRequired[str]
    r"""A registered trade name under which the business operates, if different from its legal name."""
    business_type: NotRequired[BusinessType]
    r"""The type of entity represented by this business."""
    address: NotRequired[AddressTypedDict]
    phone: NotRequired[PhoneNumberTypedDict]
    email: NotRequired[str]
    website: NotRequired[str]
    description: NotRequired[str]
    tax_id: NotRequired[TaxIDTypedDict]
    r"""An EIN (employer identification number) for the business. For sole proprietors, an SSN can be used as the EIN."""
    industry_codes: NotRequired[IndustryCodesTypedDict]
    industry: NotRequired[str]
    r"""Classification identifier for the industry. Use the [GET industries](https://docs.moov.io/api/enrichment/form-shortening/industries/get/) endpoint to retrieve an array of valid industry details for a merchant, inducing all industry field values."""
    primary_regulator: NotRequired[PrimaryRegulator]
    r"""If the business is a financial institution, this field describes its primary regulator."""


class CreateBusinessProfile(BaseModel):
    legal_business_name: Annotated[str, pydantic.Field(alias="legalBusinessName")]
    r"""The legal name under which the entity is registered."""

    doing_business_as: Annotated[
        Optional[str], pydantic.Field(alias="doingBusinessAs")
    ] = None
    r"""A registered trade name under which the business operates, if different from its legal name."""

    business_type: Annotated[
        Optional[BusinessType], pydantic.Field(alias="businessType")
    ] = None
    r"""The type of entity represented by this business."""

    address: Optional[Address] = None

    phone: Optional[PhoneNumber] = None

    email: Optional[str] = None

    website: Optional[str] = None

    description: Optional[str] = None

    tax_id: Annotated[Optional[TaxID], pydantic.Field(alias="taxID")] = None
    r"""An EIN (employer identification number) for the business. For sole proprietors, an SSN can be used as the EIN."""

    industry_codes: Annotated[
        Optional[IndustryCodes], pydantic.Field(alias="industryCodes")
    ] = None

    industry: Optional[str] = None
    r"""Classification identifier for the industry. Use the [GET industries](https://docs.moov.io/api/enrichment/form-shortening/industries/get/) endpoint to retrieve an array of valid industry details for a merchant, inducing all industry field values."""

    primary_regulator: Annotated[
        Optional[PrimaryRegulator], pydantic.Field(alias="primaryRegulator")
    ] = None
    r"""If the business is a financial institution, this field describes its primary regulator."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "doingBusinessAs",
                "businessType",
                "address",
                "phone",
                "email",
                "website",
                "description",
                "taxID",
                "industryCodes",
                "industry",
                "primaryRegulator",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    CreateBusinessProfile.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .amountdecimalvalidationerror import (
    AmountDecimalValidationError,
    AmountDecimalValidationErrorTypedDict,
)
from .createtransferlineitemoptionvalidationerror import (
    CreateTransferLineItemOptionValidationError,
    CreateTransferLineItemOptionValidationErrorTypedDict,
)
from moovio_sdk.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import Dict, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class CreateTransferLineItemValidationErrorTypedDict(TypedDict):
    product_id: NotRequired[str]
    name: NotRequired[str]
    base_price: NotRequired[AmountDecimalValidationErrorTypedDict]
    options: NotRequired[
        Dict[str, CreateTransferLineItemOptionValidationErrorTypedDict]
    ]
    quantity: NotRequired[str]
    image_i_ds: NotRequired[str]


class CreateTransferLineItemValidationError(BaseModel):
    product_id: Annotated[Optional[str], pydantic.Field(alias="productID")] = None

    name: Optional[str] = None

    base_price: Annotated[
        Optional[AmountDecimalValidationError], pydantic.Field(alias="basePrice")
    ] = None

    options: Optional[Dict[str, CreateTransferLineItemOptionValidationError]] = None

    quantity: Optional[str] = None

    image_i_ds: Annotated[Optional[str], pydantic.Field(alias="imageIDs")] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            ["productID", "name", "basePrice", "options", "quantity", "imageIDs"]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    CreateTransferLineItemValidationError.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .amountdecimal import AmountDecimal, AmountDecimalTypedDict
from datetime import datetime
from moovio_sdk.types import BaseModel
import pydantic
from typing_extensions import Annotated, TypedDict


class AdjustmentTypedDict(TypedDict):
    r"""An adjustment to a wallet."""

    adjustment_id: str
    wallet_id: str
    amount: AmountDecimalTypedDict
    created_on: datetime


class Adjustment(BaseModel):
    r"""An adjustment to a wallet."""

    adjustment_id: Annotated[str, pydantic.Field(alias="adjustmentID")]

    wallet_id: Annotated[str, pydantic.Field(alias="walletID")]

    amount: AmountDecimal

    created_on: Annotated[datetime, pydantic.Field(alias="createdOn")]


try:
    Adjustment.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from dataclasses import dataclass, field
import httpx
from moovio_sdk.models.errors import MoovError
from moovio_sdk.types import BaseModel
import pydantic
from typing import Optional
from typing_extensions import Annotated


class TerminalApplicationErrorData(BaseModel):
    platform: Optional[str] = None
    app_bundle_id: Annotated[Optional[str], pydantic.Field(alias="appBundleID")] = None
    package_name: Annotated[Optional[str], pydantic.Field(alias="packageName")] = None
    sha256_digest: Annotated[Optional[str], pydantic.Field(alias="sha256Digest")] = None
    version_code: Annotated[Optional[str], pydantic.Field(alias="versionCode")] = None


@dataclass(unsafe_hash=True)
class TerminalApplicationError(MoovError):
    data: TerminalApplicationErrorData = field(hash=False)

    def __init__(
        self,
        data: TerminalApplicationErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        message = body or raw_response.text
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class CollectionPaymentMethodType(str, Enum):
    r"""Payment methods allowed for collecting a payment."""

    APPLE_PAY = "apple-pay"
    CARD_PAYMENT = "card-payment"
    ACH_DEBIT_COLLECT = "ach-debit-collect"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from moovio_sdk.types import BaseModel
import pydantic
from typing_extensions import Annotated, TypedDict


class TransferACHAddendaRecordTypedDict(TypedDict):
    r"""An addenda record which was added to an ACH transfer"""

    record: str
    r"""The ACH addenda record, which may have masked PII"""
    is_masked: bool
    r"""Flag indicating whether or not this record has been masked"""


class TransferACHAddendaRecord(BaseModel):
    r"""An addenda record which was added to an ACH transfer"""

    record: str
    r"""The ACH addenda record, which may have masked PII"""

    is_masked: Annotated[bool, pydantic.Field(alias="isMasked")]
    r"""Flag indicating whether or not this record has been masked"""


try:
    TransferACHAddendaRecord.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from datetime import datetime
from moovio_sdk.types import BaseModel
import pydantic
from typing_extensions import Annotated, TypedDict


class AsyncTransferTypedDict(TypedDict):
    transfer_id: str
    created_on: datetime


class AsyncTransfer(BaseModel):
    transfer_id: Annotated[str, pydantic.Field(alias="transferID")]

    created_on: Annotated[datetime, pydantic.Field(alias="createdOn")]


try:
    AsyncTransfer.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class WebhookEventType(str, Enum):
    r"""The type of event that occurred."""

    WILDCARD_ = "*"
    ACCOUNT_CREATED = "account.created"
    ACCOUNT_UPDATED = "account.updated"
    ACCOUNT_DISCONNECTED = "account.disconnected"
    BALANCE_UPDATED = "balance.updated"
    BANK_ACCOUNT_CREATED = "bankAccount.created"
    BANK_ACCOUNT_UPDATED = "bankAccount.updated"
    BANK_ACCOUNT_DELETED = "bankAccount.deleted"
    CANCELLATION_CREATED = "cancellation.created"
    CANCELLATION_UPDATED = "cancellation.updated"
    CARD_AUTO_UPDATED = "card.autoUpdated"
    CAPABILITY_REQUESTED = "capability.requested"
    CAPABILITY_UPDATED = "capability.updated"
    DISPUTE_CREATED = "dispute.created"
    DISPUTE_UPDATED = "dispute.updated"
    INVOICE_CREATED = "invoice.created"
    INVOICE_UPDATED = "invoice.updated"
    NETWORK_ID_UPDATED = "networkID.updated"
    PAYMENT_METHOD_ENABLED = "paymentMethod.enabled"
    PAYMENT_METHOD_DISABLED = "paymentMethod.disabled"
    REFUND_CREATED = "refund.created"
    REFUND_UPDATED = "refund.updated"
    REPRESENTATIVE_CREATED = "representative.created"
    REPRESENTATIVE_UPDATED = "representative.updated"
    REPRESENTATIVE_DELETED = "representative.deleted"
    SWEEP_CREATED = "sweep.created"
    SWEEP_UPDATED = "sweep.updated"
    TERMINAL_APPLICATION_CREATED = "terminalApplication.created"
    TERMINAL_APPLICATION_UPDATED = "terminalApplication.updated"
    TICKET_CREATED = "ticket.created"
    TICKET_UPDATED = "ticket.updated"
    TICKET_MESSAGE_ADDED = "ticket.messageAdded"
    TRANSFER_CREATED = "transfer.created"
    TRANSFER_UPDATED = "transfer.updated"
    WALLET_CREATED = "wallet.created"
    WALLET_UPDATED = "wallet.updated"
    WALLET_TRANSACTION_UPDATED = "walletTransaction.updated"
    BILLING_STATEMENT_CREATED = "billingStatement.created"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class TerminalApplicationPlatform(str, Enum):
    r"""Platform of the terminal application."""

    UNDEFINED = "undefined"
    IOS = "ios"
    ANDROID = "android"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from moovio_sdk.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class SsnTypedDict(TypedDict):
    full: NotRequired[str]
    last_four: NotRequired[str]


class Ssn(BaseModel):
    full: Optional[str] = None

    last_four: Annotated[Optional[str], pydantic.Field(alias="lastFour")] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["full", "lastFour"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class ItinTypedDict(TypedDict):
    full: NotRequired[str]
    last_four: NotRequired[str]


class Itin(BaseModel):
    full: Optional[str] = None

    last_four: Annotated[Optional[str], pydantic.Field(alias="lastFour")] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["full", "lastFour"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class GovernmentIDTypedDict(TypedDict):
    ssn: NotRequired[SsnTypedDict]
    itin: NotRequired[ItinTypedDict]


class GovernmentID(BaseModel):
    ssn: Optional[Ssn] = None

    itin: Optional[Itin] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["ssn", "itin"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    Ssn.model_rebuild()
except NameError:
    pass
try:
    Itin.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .amountdecimal import AmountDecimal, AmountDecimalTypedDict
from .volumerange import VolumeRange, VolumeRangeTypedDict
from moovio_sdk.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class FeePropertiesTypedDict(TypedDict):
    r"""Defines the specific parameters used for fee calculation."""

    volume_ranges: List[VolumeRangeTypedDict]
    r"""Defines the volume ranges for tiered pricing models."""
    fixed_amount: NotRequired[AmountDecimalTypedDict]
    r"""A fixed fee that is applied to the amount of each transaction in the `fixed` and `blended` fee models."""
    variable_rate: NotRequired[str]
    r"""A percentage fee that is applied to the amount of each transaction in the `blended` fee model, expressed as a decimal.

    For example, 0.05% is '0.05'.
    """
    min_per_transaction: NotRequired[AmountDecimalTypedDict]
    r"""Specifies the minimum allowable spending for a single transaction, working as a transaction floor."""
    max_per_transaction: NotRequired[AmountDecimalTypedDict]
    r"""Specifies the maximum allowable spending for a single transaction, working as a transaction ceiling."""


class FeeProperties(BaseModel):
    r"""Defines the specific parameters used for fee calculation."""

    volume_ranges: Annotated[List[VolumeRange], pydantic.Field(alias="volumeRanges")]
    r"""Defines the volume ranges for tiered pricing models."""

    fixed_amount: Annotated[
        Optional[AmountDecimal], pydantic.Field(alias="fixedAmount")
    ] = None
    r"""A fixed fee that is applied to the amount of each transaction in the `fixed` and `blended` fee models."""

    variable_rate: Annotated[Optional[str], pydantic.Field(alias="variableRate")] = None
    r"""A percentage fee that is applied to the amount of each transaction in the `blended` fee model, expressed as a decimal.

    For example, 0.05% is '0.05'.
    """

    min_per_transaction: Annotated[
        Optional[AmountDecimal], pydantic.Field(alias="minPerTransaction")
    ] = None
    r"""Specifies the minimum allowable spending for a single transaction, working as a transaction floor."""

    max_per_transaction: Annotated[
        Optional[AmountDecimal], pydantic.Field(alias="maxPerTransaction")
    ] = None
    r"""Specifies the maximum allowable spending for a single transaction, working as a transaction ceiling."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            ["fixedAmount", "variableRate", "minPerTransaction", "maxPerTransaction"]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    FeeProperties.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .issuingvelocitylimit import IssuingVelocityLimit, IssuingVelocityLimitTypedDict
from moovio_sdk.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class IssuingControlsTypedDict(TypedDict):
    single_use: NotRequired[bool]
    r"""Indicates if the card is single-use. If true, the card closes after the first authorization."""
    velocity_limits: NotRequired[List[IssuingVelocityLimitTypedDict]]
    r"""Sets the spending limit per time interval. Only one limit per interval is supported."""


class IssuingControls(BaseModel):
    single_use: Annotated[Optional[bool], pydantic.Field(alias="singleUse")] = None
    r"""Indicates if the card is single-use. If true, the card closes after the first authorization."""

    velocity_limits: Annotated[
        Optional[List[IssuingVelocityLimit]], pydantic.Field(alias="velocityLimits")
    ] = None
    r"""Sets the spending limit per time interval. Only one limit per interval is supported."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["singleUse", "velocityLimits"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    IssuingControls.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class ApplicationScope(str, Enum):
    r"""A permission that the application requests on another account."""

    ACCOUNTS_READ = "accounts.read"
    ACCOUNTS_WRITE = "accounts.write"
    ANALYTICS_READ = "analytics.read"
    APPLE_PAY_MERCHANT_READ = "apple-pay-merchant.read"
    APPLE_PAY_MERCHANT_WRITE = "apple-pay-merchant.write"
    APPLE_PAY_READ = "apple-pay.read"
    APPLE_PAY_WRITE = "apple-pay.write"
    BANK_ACCOUNTS_READ = "bank-accounts.read"
    BANK_ACCOUNTS_WRITE = "bank-accounts.write"
    CAPABILITIES_READ = "capabilities.read"
    CAPABILITIES_WRITE = "capabilities.write"
    CARDS_READ = "cards.read"
    CARDS_WRITE = "cards.write"
    DOCUMENTS_READ = "documents.read"
    DOCUMENTS_WRITE = "documents.write"
    FED_READ = "fed.read"
    FILES_READ = "files.read"
    FILES_WRITE = "files.write"
    ISSUED_CARDS_READ = "issued-cards.read"
    ISSUED_CARDS_WRITE = "issued-cards.write"
    ISSUED_CARDS_READ_SECURE = "issued-cards.read-secure"
    PAYMENT_METHODS_READ = "payment-methods.read"
    PING_READ = "ping.read"
    PROFILE_ENRICHMENT_READ = "profile-enrichment.read"
    PROFILE_READ = "profile.read"
    PROFILE_WRITE = "profile.write"
    PROFILE_DISCONNECT = "profile.disconnect"
    REPRESENTATIVES_READ = "representatives.read"
    REPRESENTATIVES_WRITE = "representatives.write"
    TRANSFERS_READ = "transfers.read"
    TRANSFERS_WRITE = "transfers.write"
    WALLETS_READ = "wallets.read"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from moovio_sdk.types import BaseModel
import pydantic
from typing_extensions import Annotated, TypedDict


class PlaidLinkIntegrationTypedDict(TypedDict):
    r"""This is used by Moov.js with a Plaid reseller relationship. The details of a Plaid link integration for a linked funding source.

    You can simulate linking bank accounts with Plaid in test mode. See our [test mode](https://docs.moov.io/guides/get-started/test-mode/#plaid)
    guide for more information.

    Plaid's `sandbox` environment - (requires Plaid reseller setup with Moov). When linking a bank account to a `sandbox` account using a Plaid
    public token it will utilize Plaid's sandbox environment. The Plaid public token provided must be generated from Plaid's sandbox environment.
    Please see <a href=\"https://plaid.com/docs/api/sandbox/#sandboxpublic_tokencreate\" target=\"_blank\">Plaid's sandbox documentation</a> for more
    details.
    """

    public_token: str


class PlaidLinkIntegration(BaseModel):
    r"""This is used by Moov.js with a Plaid reseller relationship. The details of a Plaid link integration for a linked funding source.

    You can simulate linking bank accounts with Plaid in test mode. See our [test mode](https://docs.moov.io/guides/get-started/test-mode/#plaid)
    guide for more information.

    Plaid's `sandbox` environment - (requires Plaid reseller setup with Moov). When linking a bank account to a `sandbox` account using a Plaid
    public token it will utilize Plaid's sandbox environment. The Plaid public token provided must be generated from Plaid's sandbox environment.
    Please see <a href=\"https://plaid.com/docs/api/sandbox/#sandboxpublic_tokencreate\" target=\"_blank\">Plaid's sandbox documentation</a> for more
    details.
    """

    public_token: Annotated[str, pydantic.Field(alias="publicToken")]


try:
    PlaidLinkIntegration.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .amountdecimal import AmountDecimal, AmountDecimalTypedDict
from .wallettransactiontype import WalletTransactionType
from moovio_sdk.types import BaseModel
from typing_extensions import TypedDict


class SweepSubtotalTypedDict(TypedDict):
    type: WalletTransactionType
    r"""The type of wallet transaction the subtotal is for."""
    count: int
    r"""The number of transactions of this type accrued in the sweep."""
    amount: AmountDecimalTypedDict
    r"""The value of transactions of this type accrued in the sweep."""


class SweepSubtotal(BaseModel):
    type: WalletTransactionType
    r"""The type of wallet transaction the subtotal is for."""

    count: int
    r"""The number of transactions of this type accrued in the sweep."""

    amount: AmountDecimal
    r"""The value of transactions of this type accrued in the sweep."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .cardaddress import CardAddress, CardAddressTypedDict
from .cardexpiration import CardExpiration, CardExpirationTypedDict
from .e2eetoken import E2EEToken, E2EETokenTypedDict
from moovio_sdk.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class LinkCardTypedDict(TypedDict):
    card_number: str
    card_cvv: str
    expiration: CardExpirationTypedDict
    r"""The expiration date of the card or token."""
    billing_address: CardAddressTypedDict
    e2ee: NotRequired[E2EETokenTypedDict]
    r"""Wraps a compact-serialized JSON Web Encryption (JWE) token used for secure transmission of sensitive data (e.g., PCI information) through intermediaries.
    This token is encrypted using the public key from /end-to-end-keys and wraps an AES key. For details and examples, refer to our
    [GitHub repository](https://github.com/moovfinancial/moov-go/blob/main/examples/e2ee/e2ee_test.go).
    """
    holder_name: NotRequired[str]
    card_on_file: NotRequired[bool]
    merchant_account_id: NotRequired[str]
    verify_name: NotRequired[bool]


class LinkCard(BaseModel):
    card_number: Annotated[str, pydantic.Field(alias="cardNumber")]

    card_cvv: Annotated[str, pydantic.Field(alias="cardCvv")]

    expiration: CardExpiration
    r"""The expiration date of the card or token."""

    billing_address: Annotated[CardAddress, pydantic.Field(alias="billingAddress")]

    e2ee: Optional[E2EEToken] = None
    r"""Wraps a compact-serialized JSON Web Encryption (JWE) token used for secure transmission of sensitive data (e.g., PCI information) through intermediaries.
    This token is encrypted using the public key from /end-to-end-keys and wraps an AES key. For details and examples, refer to our
    [GitHub repository](https://github.com/moovfinancial/moov-go/blob/main/examples/e2ee/e2ee_test.go).
    """

    holder_name: Annotated[Optional[str], pydantic.Field(alias="holderName")] = None

    card_on_file: Annotated[Optional[bool], pydantic.Field(alias="cardOnFile")] = None

    merchant_account_id: Annotated[
        Optional[str], pydantic.Field(alias="merchantAccountID")
    ] = None

    verify_name: Annotated[Optional[bool], pydantic.Field(alias="verifyName")] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            ["e2ee", "holderName", "cardOnFile", "merchantAccountID", "verifyName"]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    LinkCard.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .amountdecimal import AmountDecimal, AmountDecimalTypedDict
from .productimagemetadata import ProductImageMetadata, ProductImageMetadataTypedDict
from .productoptiongroup import ProductOptionGroup, ProductOptionGroupTypedDict
from datetime import datetime
from moovio_sdk.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class ProductTypedDict(TypedDict):
    r"""A good or service offered by a merchant."""

    product_id: str
    r"""Unique identifier for a product."""
    title: str
    base_price: AmountDecimalTypedDict
    r"""A product's starting price, before applying modifiers."""
    created_on: datetime
    r"""The date and time when the product was added."""
    updated_on: datetime
    r"""The date and time when the product was last updated."""
    description: NotRequired[str]
    r"""A detailed description of the product.

    - Must be valid UTF-8 text
    - Supports Markdown for formatting
    - HTML is not permitted and will be rejected
    """
    option_groups: NotRequired[List[ProductOptionGroupTypedDict]]
    r"""Optional configuration options for a product, such as size or color."""
    images: NotRequired[List[ProductImageMetadataTypedDict]]
    r"""Optional images associated with the product."""
    disabled_on: NotRequired[datetime]
    r"""The date and time when the product was disabled."""


class Product(BaseModel):
    r"""A good or service offered by a merchant."""

    product_id: Annotated[str, pydantic.Field(alias="productID")]
    r"""Unique identifier for a product."""

    title: str

    base_price: Annotated[AmountDecimal, pydantic.Field(alias="basePrice")]
    r"""A product's starting price, before applying modifiers."""

    created_on: Annotated[datetime, pydantic.Field(alias="createdOn")]
    r"""The date and time when the product was added."""

    updated_on: Annotated[datetime, pydantic.Field(alias="updatedOn")]
    r"""The date and time when the product was last updated."""

    description: Optional[str] = None
    r"""A detailed description of the product.

    - Must be valid UTF-8 text
    - Supports Markdown for formatting
    - HTML is not permitted and will be rejected
    """

    option_groups: Annotated[
        Optional[List[ProductOptionGroup]], pydantic.Field(alias="optionGroups")
    ] = None
    r"""Optional configuration options for a product, such as size or color."""

    images: Optional[List[ProductImageMetadata]] = None
    r"""Optional images associated with the product."""

    disabled_on: Annotated[Optional[datetime], pydantic.Field(alias="disabledOn")] = (
        None
    )
    r"""The date and time when the product was disabled."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["description", "optionGroups", "images", "disabledOn"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    Product.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class TransferWaitFor(str, Enum):
    RAIL_RESPONSE = "rail-response"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class DomesticPushToCard(str, Enum):
    r"""Indicates which level of domestic push-to-card transfer is supported by the card, if any."""

    NOT_SUPPORTED = "not-supported"
    STANDARD = "standard"
    FAST_FUNDS = "fast-funds"
    UNKNOWN = "unknown"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .amount import Amount, AmountTypedDict
from datetime import datetime
from moovio_sdk.types import BaseModel
import pydantic
from typing_extensions import Annotated, TypedDict


class CardAcquiringDisputeTypedDict(TypedDict):
    r"""Details of a card dispute."""

    dispute_id: str
    created_on: datetime
    amount: AmountTypedDict


class CardAcquiringDispute(BaseModel):
    r"""Details of a card dispute."""

    dispute_id: Annotated[str, pydantic.Field(alias="disputeID")]

    created_on: Annotated[datetime, pydantic.Field(alias="createdOn")]

    amount: Amount


try:
    CardAcquiringDispute.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .amount import Amount, AmountTypedDict
from .createtransferdestination import (
    CreateTransferDestination,
    CreateTransferDestinationTypedDict,
)
from .createtransferlineitems import (
    CreateTransferLineItems,
    CreateTransferLineItemsTypedDict,
)
from .createtransfersource import CreateTransferSource, CreateTransferSourceTypedDict
from .facilitatorfee import FacilitatorFee, FacilitatorFeeTypedDict
from moovio_sdk.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import Dict, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class CreateTransferTypedDict(TypedDict):
    source: CreateTransferSourceTypedDict
    r"""Where funds for a transfer originate. For the source, you must include either a `paymentMethodID` or a `transferID`."""
    destination: CreateTransferDestinationTypedDict
    r"""The final stage of a transfer and the ultimate recipient of the funds."""
    amount: AmountTypedDict
    facilitator_fee: NotRequired[FacilitatorFeeTypedDict]
    r"""Total or markup fee."""
    description: NotRequired[str]
    r"""An optional description of the transfer that is used on receipts and for your own internal use."""
    metadata: NotRequired[Dict[str, str]]
    r"""Free-form key-value pair list. Useful for storing information that is not captured elsewhere."""
    sales_tax_amount: NotRequired[AmountTypedDict]
    r"""Optional sales tax amount. `transfer.amount.value` should be inclusive of any sales tax and represents the total amount charged."""
    foreign_id: NotRequired[str]
    r"""Optional alias from a foreign/external system which can be used to reference this resource."""
    line_items: NotRequired[CreateTransferLineItemsTypedDict]
    r"""An optional collection of line items for a transfer.
    When line items are provided, their total plus sales tax must equal the transfer amount.
    """


class CreateTransfer(BaseModel):
    source: CreateTransferSource
    r"""Where funds for a transfer originate. For the source, you must include either a `paymentMethodID` or a `transferID`."""

    destination: CreateTransferDestination
    r"""The final stage of a transfer and the ultimate recipient of the funds."""

    amount: Amount

    facilitator_fee: Annotated[
        Optional[FacilitatorFee], pydantic.Field(alias="facilitatorFee")
    ] = None
    r"""Total or markup fee."""

    description: Optional[str] = None
    r"""An optional description of the transfer that is used on receipts and for your own internal use."""

    metadata: Optional[Dict[str, str]] = None
    r"""Free-form key-value pair list. Useful for storing information that is not captured elsewhere."""

    sales_tax_amount: Annotated[
        Optional[Amount], pydantic.Field(alias="salesTaxAmount")
    ] = None
    r"""Optional sales tax amount. `transfer.amount.value` should be inclusive of any sales tax and represents the total amount charged."""

    foreign_id: Annotated[Optional[str], pydantic.Field(alias="foreignID")] = None
    r"""Optional alias from a foreign/external system which can be used to reference this resource."""

    line_items: Annotated[
        Optional[CreateTransferLineItems], pydantic.Field(alias="lineItems")
    ] = None
    r"""An optional collection of line items for a transfer.
    When line items are provided, their total plus sales tax must equal the transfer amount.
    """

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "facilitatorFee",
                "description",
                "metadata",
                "salesTaxAmount",
                "foreignID",
                "lineItems",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    CreateTransfer.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .bankaccountholdertype import BankAccountHolderType
from .bankaccountstatus import BankAccountStatus
from .bankaccounttype import BankAccountType
from datetime import datetime
from moovio_sdk.types import BaseModel
import pydantic
from typing_extensions import Annotated, TypedDict


class PaymentMethodsBankAccountTypedDict(TypedDict):
    r"""A bank account as contained within a payment method."""

    bank_account_id: str
    fingerprint: str
    r"""Once the bank account is linked, we don't reveal the full bank account number.

    The fingerprint acts as a way to identify whether two linked bank accounts are the same.
    """
    status: BankAccountStatus
    holder_name: str
    holder_type: BankAccountHolderType
    r"""The type of holder on a funding source."""
    bank_name: str
    bank_account_type: BankAccountType
    r"""The bank account type."""
    routing_number: str
    last_four_account_number: str
    updated_on: datetime


class PaymentMethodsBankAccount(BaseModel):
    r"""A bank account as contained within a payment method."""

    bank_account_id: Annotated[str, pydantic.Field(alias="bankAccountID")]

    fingerprint: str
    r"""Once the bank account is linked, we don't reveal the full bank account number.

    The fingerprint acts as a way to identify whether two linked bank accounts are the same.
    """

    status: BankAccountStatus

    holder_name: Annotated[str, pydantic.Field(alias="holderName")]

    holder_type: Annotated[BankAccountHolderType, pydantic.Field(alias="holderType")]
    r"""The type of holder on a funding source."""

    bank_name: Annotated[str, pydantic.Field(alias="bankName")]

    bank_account_type: Annotated[
        BankAccountType, pydantic.Field(alias="bankAccountType")
    ]
    r"""The bank account type."""

    routing_number: Annotated[str, pydantic.Field(alias="routingNumber")]

    last_four_account_number: Annotated[
        str, pydantic.Field(alias="lastFourAccountNumber")
    ]

    updated_on: Annotated[datetime, pydantic.Field(alias="updatedOn")]


try:
    PaymentMethodsBankAccount.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .address import Address, AddressTypedDict
from .enrichedindustrycodes import EnrichedIndustryCodes, EnrichedIndustryCodesTypedDict
from .phonenumber import PhoneNumber, PhoneNumberTypedDict
from moovio_sdk.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class EnrichedBusinessProfileTypedDict(TypedDict):
    address: NotRequired[AddressTypedDict]
    email: NotRequired[str]
    industry_codes: NotRequired[EnrichedIndustryCodesTypedDict]
    legal_business_name: NotRequired[str]
    phone: NotRequired[PhoneNumberTypedDict]
    website: NotRequired[str]


class EnrichedBusinessProfile(BaseModel):
    address: Optional[Address] = None

    email: Optional[str] = None

    industry_codes: Annotated[
        Optional[EnrichedIndustryCodes], pydantic.Field(alias="industryCodes")
    ] = None

    legal_business_name: Annotated[
        Optional[str], pydantic.Field(alias="legalBusinessName")
    ] = None

    phone: Optional[PhoneNumber] = None

    website: Optional[str] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "address",
                "email",
                "industryCodes",
                "legalBusinessName",
                "phone",
                "website",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    EnrichedBusinessProfile.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from moovio_sdk.types import BaseModel
from typing_extensions import TypedDict


class CompleteBankAccountVerificationTypedDict(TypedDict):
    code: str
    r"""Code provided by user from their bank account transactions"""


class CompleteBankAccountVerification(BaseModel):
    code: str
    r"""Code provided by user from their bank account transactions"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .fednowservices import FedNowServices, FedNowServicesTypedDict
from moovio_sdk.types import BaseModel
import pydantic
from typing_extensions import Annotated, TypedDict


class FedNowInstitutionTypedDict(TypedDict):
    name: str
    r"""Name of the financial institution."""
    routing_number: str
    services: FedNowServicesTypedDict


class FedNowInstitution(BaseModel):
    name: str
    r"""Name of the financial institution."""

    routing_number: Annotated[str, pydantic.Field(alias="routingNumber")]

    services: FedNowServices


try:
    FedNowInstitution.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from moovio_sdk.types import BaseModel
from typing_extensions import TypedDict


class E2EETokenTypedDict(TypedDict):
    r"""Wraps a compact-serialized JSON Web Encryption (JWE) token used for secure transmission of sensitive data (e.g., PCI information) through intermediaries.
    This token is encrypted using the public key from /end-to-end-keys and wraps an AES key. For details and examples, refer to our
    [GitHub repository](https://github.com/moovfinancial/moov-go/blob/main/examples/e2ee/e2ee_test.go).
    """

    token: str
    r"""An [RFC](https://datatracker.ietf.org/doc/html/rfc7516) compact-serialized JSON Web Encryption (JWE) token."""


class E2EEToken(BaseModel):
    r"""Wraps a compact-serialized JSON Web Encryption (JWE) token used for secure transmission of sensitive data (e.g., PCI information) through intermediaries.
    This token is encrypted using the public key from /end-to-end-keys and wraps an AES key. For details and examples, refer to our
    [GitHub repository](https://github.com/moovfinancial/moov-go/blob/main/examples/e2ee/e2ee_test.go).
    """

    token: str
    r"""An [RFC](https://datatracker.ietf.org/doc/html/rfc7516) compact-serialized JSON Web Encryption (JWE) token."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from dataclasses import dataclass, field
import httpx
from moovio_sdk.models.components import (
    cardexpirationerror as components_cardexpirationerror,
    createauthorizedusererror as components_createauthorizedusererror,
    issuingcontrolserror as components_issuingcontrolserror,
)
from moovio_sdk.models.errors import MoovError
from moovio_sdk.types import BaseModel
import pydantic
from typing import Optional
from typing_extensions import Annotated


class RequestCardErrorData(BaseModel):
    funding_wallet_id: Annotated[
        Optional[str], pydantic.Field(alias="fundingWalletID")
    ] = None
    form_factor: Annotated[Optional[str], pydantic.Field(alias="formFactor")] = None
    authorized_user: Annotated[
        Optional[components_createauthorizedusererror.CreateAuthorizedUserError],
        pydantic.Field(alias="authorizedUser"),
    ] = None
    memo: Optional[str] = None
    expiration: Optional[components_cardexpirationerror.CardExpirationError] = None
    controls: Optional[components_issuingcontrolserror.IssuingControlsError] = None


@dataclass(unsafe_hash=True)
class RequestCardError(MoovError):
    data: RequestCardErrorData = field(hash=False)

    def __init__(
        self,
        data: RequestCardErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        message = body or raw_response.text
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from moovio_sdk.types import BaseModel
import pydantic
from typing_extensions import Annotated, TypedDict


class MonthlyPlatformFeeTypedDict(TypedDict):
    r"""Fixed recurring amount paid in the billing period regardless of usage."""

    currency: str
    r"""A 3-letter ISO 4217 currency code."""
    value_decimal: str
    r"""A decimal-formatted numerical string that represents up to 9 decimal place precision.

    For example, $12.987654321 is '12.987654321'.
    """


class MonthlyPlatformFee(BaseModel):
    r"""Fixed recurring amount paid in the billing period regardless of usage."""

    currency: str
    r"""A 3-letter ISO 4217 currency code."""

    value_decimal: Annotated[str, pydantic.Field(alias="valueDecimal")]
    r"""A decimal-formatted numerical string that represents up to 9 decimal place precision.

    For example, $12.987654321 is '12.987654321'.
    """


try:
    MonthlyPlatformFee.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .address import Address, AddressTypedDict
from .phonenumber import PhoneNumber, PhoneNumberTypedDict
from moovio_sdk.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class CustomerSupportTypedDict(TypedDict):
    r"""User-provided information that can be displayed on credit card transactions for customers to use when
    contacting a customer support team. This data is only allowed on a business account.
    """

    phone: NotRequired[PhoneNumberTypedDict]
    email: NotRequired[str]
    address: NotRequired[AddressTypedDict]
    website: NotRequired[str]


class CustomerSupport(BaseModel):
    r"""User-provided information that can be displayed on credit card transactions for customers to use when
    contacting a customer support team. This data is only allowed on a business account.
    """

    phone: Optional[PhoneNumber] = None

    email: Optional[str] = None

    address: Optional[Address] = None

    website: Optional[str] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["phone", "email", "address", "website"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from datetime import datetime
from moovio_sdk.models.components import (
    dispute as components_dispute,
    disputestatus as components_disputestatus,
)
from moovio_sdk.types import BaseModel, UNSET_SENTINEL
from moovio_sdk.utils import (
    FieldMetadata,
    HeaderMetadata,
    PathParamMetadata,
    QueryParamMetadata,
)
import pydantic
from pydantic import model_serializer
from typing import Dict, List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class ListDisputesGlobalsTypedDict(TypedDict):
    x_moov_version: NotRequired[str]
    r"""Specify an API version.

    API versioning follows the format `vYYYY.QQ.BB`, where
    - `YYYY` is the year
    - `QQ` is the two-digit month for the first month of the quarter (e.g., 01, 04, 07, 10)
    - `BB` is the build number, starting at `.01`, for subsequent builds in the same quarter.
    - For example, `v2024.01.00` is the initial release of the first quarter of 2024.

    The `dev` version represents the most recent development state. It may include breaking changes and should be treated as a beta release.
    When no version is specified, the API defaults to `v2024.01.00`.
    """


class ListDisputesGlobals(BaseModel):
    x_moov_version: Annotated[
        Optional[str],
        pydantic.Field(alias="X-Moov-Version"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""Specify an API version.

    API versioning follows the format `vYYYY.QQ.BB`, where
    - `YYYY` is the year
    - `QQ` is the two-digit month for the first month of the quarter (e.g., 01, 04, 07, 10)
    - `BB` is the build number, starting at `.01`, for subsequent builds in the same quarter.
    - For example, `v2024.01.00` is the initial release of the first quarter of 2024.

    The `dev` version represents the most recent development state. It may include breaking changes and should be treated as a beta release.
    When no version is specified, the API defaults to `v2024.01.00`.
    """

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["X-Moov-Version"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class ListDisputesRequestTypedDict(TypedDict):
    account_id: str
    skip: NotRequired[int]
    count: NotRequired[int]
    start_date_time: NotRequired[datetime]
    r"""Optional date-time parameter to filter all disputes created on and after the provided date and time."""
    end_date_time: NotRequired[datetime]
    r"""Optional date-time parameter to filter all disputes created on and before the provided date and time."""
    respond_start_date_time: NotRequired[datetime]
    r"""Optional date-time which exclusively filters all disputes with respond by before this date-time."""
    respond_end_date_time: NotRequired[datetime]
    r"""Optional date-time which exclusively filters all disputes with respond by before this date-time."""
    status: NotRequired[components_disputestatus.DisputeStatus]
    r"""Optional dispute status by which to filter the disputes."""
    merchant_account_id: NotRequired[str]
    r"""Optional parameter to filter by merchant account ID."""
    cardholder_account_id: NotRequired[str]
    r"""Optional parameter to filter by cardholder account ID."""
    dispute_i_ds: NotRequired[List[str]]
    r"""Optional parameter to filter by a comma separated list of dispute IDs."""
    transfer_i_ds: NotRequired[List[str]]
    r"""Optional parameter to filter by a comma separated list of transfer IDs."""
    order_by: NotRequired[str]


class ListDisputesRequest(BaseModel):
    account_id: Annotated[
        str,
        pydantic.Field(alias="accountID"),
        FieldMetadata(path=PathParamMetadata(style="simple", explode=False)),
    ]

    skip: Annotated[
        Optional[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=False)),
    ] = None

    count: Annotated[
        Optional[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=False)),
    ] = None

    start_date_time: Annotated[
        Optional[datetime],
        pydantic.Field(alias="startDateTime"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=False)),
    ] = None
    r"""Optional date-time parameter to filter all disputes created on and after the provided date and time."""

    end_date_time: Annotated[
        Optional[datetime],
        pydantic.Field(alias="endDateTime"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=False)),
    ] = None
    r"""Optional date-time parameter to filter all disputes created on and before the provided date and time."""

    respond_start_date_time: Annotated[
        Optional[datetime],
        pydantic.Field(alias="respondStartDateTime"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=False)),
    ] = None
    r"""Optional date-time which exclusively filters all disputes with respond by before this date-time."""

    respond_end_date_time: Annotated[
        Optional[datetime],
        pydantic.Field(alias="respondEndDateTime"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=False)),
    ] = None
    r"""Optional date-time which exclusively filters all disputes with respond by before this date-time."""

    status: Annotated[
        Optional[components_disputestatus.DisputeStatus],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=False)),
    ] = None
    r"""Optional dispute status by which to filter the disputes."""

    merchant_account_id: Annotated[
        Optional[str],
        pydantic.Field(alias="merchantAccountID"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=False)),
    ] = None
    r"""Optional parameter to filter by merchant account ID."""

    cardholder_account_id: Annotated[
        Optional[str],
        pydantic.Field(alias="cardholderAccountID"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=False)),
    ] = None
    r"""Optional parameter to filter by cardholder account ID."""

    dispute_i_ds: Annotated[
        Optional[List[str]],
        pydantic.Field(alias="disputeIDs"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=False)),
    ] = None
    r"""Optional parameter to filter by a comma separated list of dispute IDs."""

    transfer_i_ds: Annotated[
        Optional[List[str]],
        pydantic.Field(alias="transferIDs"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=False)),
    ] = None
    r"""Optional parameter to filter by a comma separated list of transfer IDs."""

    order_by: Annotated[
        Optional[str],
        pydantic.Field(alias="orderBy"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=False)),
    ] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "skip",
                "count",
                "startDateTime",
                "endDateTime",
                "respondStartDateTime",
                "respondEndDateTime",
                "status",
                "merchantAccountID",
                "cardholderAccountID",
                "disputeIDs",
                "transferIDs",
                "orderBy",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class ListDisputesResponseTypedDict(TypedDict):
    headers: Dict[str, List[str]]
    result: List[components_dispute.DisputeTypedDict]


class ListDisputesResponse(BaseModel):
    headers: Dict[str, List[str]]

    result: List[components_dispute.Dispute]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from moovio_sdk.types import BaseModel
import pydantic
from typing_extensions import Annotated, TypedDict


class QRCodeTypedDict(TypedDict):
    qr_code: str
    r"""A base64-encoded PNG image of the QR code."""


class QRCode(BaseModel):
    qr_code: Annotated[str, pydantic.Field(alias="qrCode")]
    r"""A base64-encoded PNG image of the QR code."""


try:
    QRCode.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .bankaccountholdertype import BankAccountHolderType
from .bankaccounttype import BankAccountType
from moovio_sdk.types import BaseModel
import pydantic
from typing_extensions import Annotated, TypedDict


class BankAccountIntegrationTypedDict(TypedDict):
    holder_name: str
    holder_type: BankAccountHolderType
    r"""The type of holder on a funding source."""
    account_number: str
    bank_account_type: BankAccountType
    r"""The bank account type."""
    routing_number: str


class BankAccountIntegration(BaseModel):
    holder_name: Annotated[str, pydantic.Field(alias="holderName")]

    holder_type: Annotated[BankAccountHolderType, pydantic.Field(alias="holderType")]
    r"""The type of holder on a funding source."""

    account_number: Annotated[str, pydantic.Field(alias="accountNumber")]

    bank_account_type: Annotated[
        BankAccountType, pydantic.Field(alias="bankAccountType")
    ]
    r"""The bank account type."""

    routing_number: Annotated[str, pydantic.Field(alias="routingNumber")]


try:
    BankAccountIntegration.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from moovio_sdk.types import BaseModel
import pydantic
from typing_extensions import Annotated, TypedDict


class CardVolumeDistributionTypedDict(TypedDict):
    ecommerce_percentage: int
    card_present_percentage: int
    mail_or_phone_percentage: int
    debt_repayment_percentage: int


class CardVolumeDistribution(BaseModel):
    ecommerce_percentage: Annotated[int, pydantic.Field(alias="ecommercePercentage")]

    card_present_percentage: Annotated[
        int, pydantic.Field(alias="cardPresentPercentage")
    ]

    mail_or_phone_percentage: Annotated[
        int, pydantic.Field(alias="mailOrPhonePercentage")
    ]

    debt_repayment_percentage: Annotated[
        int, pydantic.Field(alias="debtRepaymentPercentage")
    ]


try:
    CardVolumeDistribution.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from moovio_sdk.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class IssuingMerchantDataTypedDict(TypedDict):
    network_id: str
    r"""External identifier used to identify the merchant with the card brand."""
    country: str
    r"""Two-letter country code."""
    mcc: str
    r"""The Merchant Category Code."""
    name: NotRequired[str]
    r"""Name of the merchant."""
    city: NotRequired[str]
    r"""The merchant's location."""
    postal_code: NotRequired[str]
    r"""The merchant's five-digit postal code."""
    state: NotRequired[str]
    r"""The merchant's two-letter state abbreviation."""


class IssuingMerchantData(BaseModel):
    network_id: Annotated[str, pydantic.Field(alias="networkID")]
    r"""External identifier used to identify the merchant with the card brand."""

    country: str
    r"""Two-letter country code."""

    mcc: str
    r"""The Merchant Category Code."""

    name: Optional[str] = None
    r"""Name of the merchant."""

    city: Optional[str] = None
    r"""The merchant's location."""

    postal_code: Annotated[Optional[str], pydantic.Field(alias="postalCode")] = None
    r"""The merchant's five-digit postal code."""

    state: Optional[str] = None
    r"""The merchant's two-letter state abbreviation."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["name", "city", "postalCode", "state"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    IssuingMerchantData.model_rebuild()
except NameError:
    pass
